// Version 2.0
// Compiler Borland Developer Studio 2006
// This software is free for non commercial use,
// for commercial use, please contact the author.

unit PICUSBClass;

interface

uses
  SysUtils, Classes, usbAPI, windows;
Const vid_pid  : PCHAR  = 'vid_04d8&pid_000c' + #0;    // Default Demo Application Firmware
      out_pipe : PCHAR = '\MCHP_EP1' + #0;
      in_pipe  : PCHAR = '\MCHP_EP1' + #0;

      READ_VERSION      = $00;
      WRITE_READ_DATA   = $04;
      SET_LED           = $06;
      SET_PIC_LED       = $30; //'0'
      GET_PIC_TEMP      = $54; //'T'
      GET_PIC_LIGHT     = $4C; //'L'

      UsbBufSize = 64;

type
  PByteBuffer = ^TByteBuffer;
  TByteBuffer = Array[0..63] of Byte;
                            
  TPICUSB = class(TComponent)
  private
    { Private declarations }
    myOutPipe : THANDLE;
    myInPipe  : THANDLE;
    isLedOn   : Boolean;
    pgaRange : Byte;
    LightValue,
    DayChannel0Value,
    DayChannel1value  : Word;
    SndDelay  : Word;
    RcvDelay  : Word;
    function isConnected: Boolean;
    function getTemperature: Single;
    function getLight: Word;
    function getDayChannel0: Word;
    function getDayChannel1: Word;
    function getRange: Byte;
    function getVersion: String;
    procedure SetLed(LedOn: Boolean);
    function getLedStatus: Boolean;
    procedure setSendDelay(Value: Word);
    function getSendDelay: Word;
    procedure setRcvDelay(Value: Word);
    function getRcvDelay: Word;
    Function SendReceivePacket(SendData :PByteBuffer; SendLength : DWORD ; ReceiveData : PByteBuffer; var ReceiveLength : DWORD):Boolean;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    function Connect: Boolean;
    function Disconnect: Boolean;
  published
    { Published declarations }
    property Connected: Boolean read isConnected;
    property Temperature: Single read getTemperature;
    property Light: Word read getLight;
    property Range: Byte read getRange;
    property DayChannel0: Word read getDayChannel0;
    property DayChannel1: Word read getDayChannel1;
    property Version: String read getVersion;
    property LedOn: Boolean read getLedStatus write SetLed;
    property SendDelay: Word read getSendDelay write setSendDelay;
    property ReceiveDelay: Word read getRcvDelay write setRcvDelay;
  end;

{procedure Register; }

implementation

Constructor TPICUSB.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  myOutPipe := INVALID_HANDLE_VALUE;
  myInPipe := INVALID_HANDLE_VALUE;
  isLedOn := False;
  pgaRange := 0;
  SndDelay := 50;
  RcvDelay := 50;
  LightValue := 0;
  DayChannel0Value := 0;
  DayChannel1Value := 0;
end;

Destructor TPICUSB.Destroy;
begin
  if isConnected then Disconnect;
  inherited;
end;

function TPICUSB.isConnected: Boolean;
begin
  If (myOutPipe = INVALID_HANDLE_VALUE) or (myInPipe = INVALID_HANDLE_VALUE) then
    result := False
  else
    result := True;
end;


function TPICUSB.Connect: Boolean;
Var Selection : DWORD;
begin
  if not isConnected then begin
    Selection:=0; // according to our firmware, only index 0 is available!
    try
      myOutPipe := MPUSBOpen(selection,vid_pid, out_pipe, MP_WRITE, 0);
      myInPipe  := MPUSBOpen(selection,vid_pid, in_pipe,  MP_READ,  0);
    except
      MPUSBClose(myOutPipe);
      MPUSBClose(myInPipe);
      myOutPipe := INVALID_HANDLE_VALUE;
      myInPipe := INVALID_HANDLE_VALUE;
    end;
  end;
  result := isConnected;
end;

function TPICUSB.Disconnect: Boolean;
begin
  if isConnected then
  begin
    try
      MPUSBClose(myOutPipe);
      MPUSBClose(myInPipe);
    finally
      myOutPipe := INVALID_HANDLE_VALUE;
      myInPipe := INVALID_HANDLE_VALUE;
    end;
  end;
  LightValue := 0;
  DayChannel0Value := 0;
  DayChannel1Value := 0;
  pgaRange := 0;
  result := True;
end;


function TPICUSB.getTemperature: single;
var raw: word;
  temp: smallint;
  RecvLength  : DWORD;
  send_buf    : TByteBuffer;
  receive_buf : TByteBuffer;
begin
  send_buf[0] := GET_PIC_TEMP;
  RecvLength := 2;
  if SendReceivePacket(@send_buf,1,@receive_buf,RecvLength) Then
  Begin
    raw := 256 * receive_buf[1] + receive_buf[0];
    temp := raw;
    temp := temp SHR 3;// remove status bits ?!
    temp := 625 * temp DIV 1000;   // Umrechnung von 1/16 zu 1/10
    //result := temp DIV 10 + 0.1*(temp mod 10);
    result := temp/10.0;
  end else
  begin
    if SndDelay < 1000 then Inc(SndDelay);
    if RcvDelay < 1000 then Inc(RcvDelay);
    result := -273.15;
  end;
end;

function TPICUSB.getLight: Word;
var
  RecvLength  : DWORD;
  send_buf    : TByteBuffer;
  receive_buf : TByteBuffer;
begin
  send_buf[0] := GET_PIC_LIGHT;
  RecvLength := 7;
  if SendReceivePacket(@send_buf,1,@receive_buf,RecvLength) Then
  Begin
    LightValue := 256 * receive_buf[1] + receive_buf[0];
    pgaRange := receive_buf[2];
    DayChannel0Value := 256 * receive_buf[4] + receive_buf[3];
    DayChannel1Value := 256 * receive_buf[6] + receive_buf[5];
    result := Lightvalue;
  end else
  begin
    if SndDelay < 1000 then Inc(SndDelay);
    if RcvDelay < 1000 then Inc(RcvDelay);
    LightValue := 0;
    DayChannel0Value := 0;
    DayChannel1Value := 0;
    pgaRange := 0;
    result := 0;
  end;

end;

function TPICUSB.getLedStatus;
begin
  result := isLedOn;
end;

function TPICUSB.getRange;
begin
  result := pgaRange;
end;

function TPICUSB.getDayChannel0;
begin
  result := DayChannel0Value;
end;
function TPICUSB.getDayChannel1;
begin
  result := DayChannel1Value;
end;

function TPICUSB.getSendDelay;
begin
  result := SndDelay;
end;

function TPICUSB.getRcvDelay;
begin
  result := RcvDelay;
end;

procedure TPICUSB.setSendDelay(Value: Word);
begin
  SndDelay := Value;
end;

procedure TPICUSB.setRcvDelay(Value: Word);
begin
  RcvDelay := Value;
end;

procedure TPICUSB.SetLed(LedOn: Boolean);
var
  RecvLength  : DWORD;
  send_buf    : TByteBuffer;
  receive_buf : TByteBuffer;
begin
  send_buf[0] := SET_PIC_LED;
  if LedOn then send_buf[1] := 1 else send_buf[0] := 0;
  RecvLength := 2;
  if SendReceivePacket(@send_buf,2,@receive_buf,RecvLength) Then
  Begin
    isLedOn := LedOn;
  end else
  begin
    isLedOn := False;
  end;
end;

function TPICUSB.getVersion:String;
var temp:DWORD;
    versionInfo : Array[0..3] of Byte;
begin
  temp := MPUSBGetDLLVersion();
  move(temp,VersionInfo,sizeof(VersionInfo));
  result := intTOStr(VersionInfo[0]) +  '.' + intTOStr(VersionInfo[1]) +  '.' + intTOStr(VersionInfo[2]) +  '.'+ intTOStr(VersionInfo[3]);
end;

Function TPICUSB.SendReceivePacket(SendData :PByteBuffer; SendLength : DWORD ; ReceiveData : PByteBuffer; var ReceiveLength : DWORD):Boolean;
var
   SentDataLength:  DWORD ;
   ExpectedReceiveLength: DWORD;
Begin
  ExpectedReceiveLength := ReceiveLength;
  if isConnected then
  Begin
    if MPUSBWrite(myOutPipe,SendData,SendLength,@SentDataLength,SndDelay) <> 0 then
    Begin
      if(MPUSBRead(myInPipe,ReceiveData, ExpectedReceiveLength,@ReceiveLength,RcvDelay)) <> 0 then
      Begin
        if (ReceiveLength = ExpectedReceiveLength) Then
        Begin
          result:=True;   // Success!
          Exit;
        End
      End //Read
    End //Write
  End; //invalid_Handle
  result:=False;  // Operation Failed
End;//end SendReceivePacket

{
procedure Register;
begin
  RegisterComponents('Samples', [TPICUSB]);
end;
}
end.
