unit UMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, PICUSBClass, ExtCtrls, Math, Menus, DateUtils;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Connect1: TMenuItem;
    Disconnect1: TMenuItem;
    N1: TMenuItem;
    Quit1: TMenuItem;
    Window1: TMenuItem;
    ile1: TMenuItem;
    Cascade1: TMenuItem;
    MISettings: TMenuItem;
    procedure MISettingsClick(Sender: TObject);
    procedure Cascade1Click(Sender: TObject);
    procedure ile1Click(Sender: TObject);
    procedure Quit1Click(Sender: TObject);
    procedure Disconnect1Click(Sender: TObject);
    procedure Connect1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure IdleHandler(Sender: TObject; var Done: Boolean);

  private
    { Private declarations }
    MySensor: TPICUSB;
    Timestamp,LastTime: TDateTime;

  public
    { Public declarations }
    Flux: Longint;
  end;

var
  Form1: TForm1;

implementation

uses UForm2, UForm3, UForm4, UForm5, USettings;

{$R *.dfm}

procedure TForm1.Cascade1Click(Sender: TObject);
begin
  Form1.Cascade;
end;

procedure TForm1.Connect1Click(Sender: TObject);
begin
  Application.OnIdle := IdleHandler;
end;

procedure TForm1.Disconnect1Click(Sender: TObject);
begin
  Application.OnIdle := nil;

end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  MySensor := TPICUSB.Create(Form1);
  Application.OnIdle := IdleHandler;
  DecimalSeparator := '.';
  LastTime := Now;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  MySensor.Destroy;
end;

procedure TForm1.IdleHandler(Sender: TObject; var Done: Boolean);
var Light,
    Range,DC0,DC1: Word;
    Ratio, Lux, Temperature, k0, k1: Single;
    F: Textfile;
    Filename: TFilename;
begin
  Flux := 0;
  Lux := 0;
  TimeStamp := now;
  if MySensor.Connected then begin
    Light := MySensor.Light;
    Range := MySensor.Range;
    Temperature := MySensor.Temperature;
    case Range of
      1: Flux := Light * 120;
      2: Flux := Light * 8;
      3: Flux := Light * 4;
      4: Flux := Light * 2;
      5: Flux := Light * 1;
    else
      Flux := 0;
    end;
    DC0 := MySensor.DayChannel0;
    DC1 := MySensor.DayChannel1;
    if DC0 > 0 then Ratio := DC1 / DC0 else Ratio := 1.30;
    k0 := 0.0; k1 := 0.0;
    if (Ratio <= 0.5) then begin
      k0 := 0.0304;
      k1 := 0.062*Power(Ratio,1.4);
    end;
    if (0.50 < Ratio) and (Ratio <= 0.61) then begin
      k0 := 0.0224;
      k1 := 0.031;
    end;
    if (0.61 < Ratio) and (Ratio <= 0.80) then begin
      k0 := 0.0128;
      k1 := 0.0153;
    end;
    if (0.80 < Ratio) and (Ratio <= 1.30) then begin
      k0 := 0.00146;
      k1 := 0.00112;
    end;
    if (1.30 < Ratio) then Lux := 0.0;
    Lux := k0*DC0 - k1*DC1;
    FTemperature.UpdateChart(TimeStamp,Temperature);
    FFlux.UpdateChart(TimeStamp,Flux);
    FCalibration.UpdateChart(Flux,lux);
    Form5.UpdateChart(TimeStamp,lux,DC0*k0,DC1*k1);
    Application.Title := 'Luxmeter '+IntToStr(Round(Lux))+ 'lux ' + IntToStr(Round(Temperature))+'C';
    Filename := '.\'+FormatDateTime('yyyymmdd',TimeStamp)+'.txt';

    if (SecondsBetween(TimeStamp,Lasttime) >= 1) then begin
      AssignFile(F,Filename);
      if FileExists(Filename) then Append(F) else Rewrite(F);
      WriteLn(F,FormatDateTime('yyyy-mm-dd hh:nn:ss',TimeStamp)+
        Format(' %05.1f %07d %1d %8.2f',[Temperature,Flux,Range,lux]));
      CloseFile(F);
      Lasttime := Timestamp;
    end;
  end
  else MySensor.Connect;

{   PBRange.Position := Range;
      PBLight.Position := Light;
      PBTemp.Position := Round(Temperature);

      if Light > 0 then
    end;
  end else MySensor.Active := True;
  UpdateStatusBar;}
  Done := True;
end;


procedure TForm1.ile1Click(Sender: TObject);
begin
  Form1.Tile;
end;

procedure TForm1.MISettingsClick(Sender: TObject);
begin
  FSettings.Show;
end;

procedure TForm1.Quit1Click(Sender: TObject);
begin
  Close;
end;

end.
